﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Microsoft.Practices.Unity;
using Workshops.Model;
using Workshops.Model.UnitsOfWork;
using Workshops.Model.Validation;
using Workshops.ViewModels.Navigation;

namespace Workshops.ViewModels
{
    public class EditWorkshopViewModel : ViewModelBase, IViewModel
    {
        private Workshop _workshop;
        private Attendee _selectedAttendee;
        private string _newAttendeeFirstName;
        private string _newAttendeeLastName;
        private ObservableCollection<Attendee> _attendees;

        [Dependency]
        public INavigationService NavigationService
        {
            get;
            set;
        }

        [Dependency]
        public IUnitOfWorkFactory UnitOfWorkFactory
        {
            get;
            set;
        }

        [Dependency]
        public IValidationManager ValidationManager
        {
            get;
            set;
        }

        public Workshop Workshop
        {
            get
            {
                return _workshop;
            }
            set
            {
                _workshop = value;
                RaisePropertyChanged("Workshop");
            }
        }

        public ObservableCollection<Attendee> Attendees
        {
            get
            {
                return _attendees;
            }
            set
            {
                _attendees = value;
                RaisePropertyChanged("Attendees");
            }
        }

        public Attendee SelectedAttendee
        {
            get
            {
                return _selectedAttendee;
            }
            set
            {
                _selectedAttendee = value;
                RaisePropertyChanged("SelectedAttendee");
            }
        }

        public string NewAttendeeFirstName
        {
            get
            {
                return _newAttendeeFirstName;
            }
            set
            {
                _newAttendeeFirstName = value;
                RaisePropertyChanged("NewAttendeeFirstName");
            }
        }

        public string NewAttendeeLastName
        {
            get
            {
                return _newAttendeeLastName;
            }
            set
            {
                _newAttendeeLastName = value;
                RaisePropertyChanged("NewAttendeeLastName");
            }
        }

        public ICommand AddNewAttendeeCommand
        {
            get;
            private set;
        }

        public ICommand DeleteSelectedAttendeeCommand
        {
            get;
            private set;
        }

        public ICommand SaveCommand
        {
            get;
            private set;
        }

        public ICommand CancelCommand
        {
            get;
            private set;
        }

        public EditWorkshopViewModel(Workshop workshop)
        {
            AddNewAttendeeCommand = new RelayCommand(AddNewAttendee);
            DeleteSelectedAttendeeCommand = new RelayCommand(DeleteSelectedAttendee);
            SaveCommand = new RelayCommand(Save);
            CancelCommand = new RelayCommand(Cancel);

            // prepare workshop
            if (workshop == null)
            {
                throw new ArgumentNullException("workshop");
            }
            Workshop = workshop;

            // prepare attendees
            Attendees = new ObservableCollection<Attendee>(Workshop.Attendees);
        }
        
        public void Initialize()
        {
            // nothing here (yet)
        }

        private void AddNewAttendee()
        {
            // create attendee
            var newAttendee = new Attendee();
            newAttendee.FirstName = NewAttendeeFirstName;
            newAttendee.LastName = NewAttendeeLastName;

            // validate
            var validationResult = ValidationManager.Validate(newAttendee);
            if (validationResult.ResultType != ValidationResultType.Valid)
            {
                MessageBox.Show(validationResult.Message);
                return;
            }
            
            // add to list
            Attendees.Add(newAttendee);

            // clean display
            NewAttendeeFirstName = string.Empty;
            NewAttendeeLastName = string.Empty;
        }

        private void DeleteSelectedAttendee()
        {
            if (SelectedAttendee == null)
            {
                MessageBox.Show("Please select an attendee you want to remove.");
                return;
            }

            // remove from list
            Attendees.Remove(SelectedAttendee);
        }

        private void Save()
        {
            var validationResult = ValidationManager.Validate(Workshop);
            if (validationResult.ResultType != ValidationResultType.Valid)
            {
                MessageBox.Show(validationResult.Message);
                return;
            }

            // update attendees in workshop
            Workshop.Attendees = Attendees;

            using (var unitOfWork = UnitOfWorkFactory.CreateUnitOfWork())
            {
                if (Workshop.Id == 0)
                {
                    // new workshop
                    unitOfWork.WorkshopsRepository.AddWorkshop(Workshop);
                }
                else
                {
                    unitOfWork.WorkshopsRepository.EditWorkshop(Workshop);
                }

                unitOfWork.SaveChanges();
            }

            NavigateToOverviewPage();
        }

        private void Cancel()
        {
            if (MessageBox.Show("Are you sure?", "Cancel", MessageBoxButton.YesNo) != MessageBoxResult.Yes)
            {
                return;
            }

            NavigateToOverviewPage();
        }

        private void NavigateToOverviewPage()
        {
            NavigationService.Navigate<DisplayWorkshopsViewModel>();
        }
    }
}
