﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using Microsoft.Practices.Unity;
using Workshops.Model.UnitsOfWork;
using Workshops.Model.Validation;
using Workshops.ViewModels;
using Workshops.ViewModels.Factory;
using Workshops.ViewModels.Navigation;
using Workshops.Views;

namespace Workshops
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        static App()
        {
            // initialize data directory
            var dataDirectory = AppDomain.CurrentDomain.BaseDirectory.Replace("bin\\Debug\\", string.Empty);
            AppDomain.CurrentDomain.SetData("DataDirectory", dataDirectory);    
        }

        public ShellViewModel ShellViewModel
        {
            get;
            private set;
        }

        protected override void OnStartup(StartupEventArgs e)
        {
            base.OnStartup(e);

            var unityContainer = InitializeDependencyContainer();
        
            // create the main windows
            var mainWindow = new MainWindow();
            mainWindow.DataContext = unityContainer.Resolve<INavigationFrame>();

            // initialize 
            MainWindow = mainWindow;
            mainWindow.Show();

            // navigate
            var navigationService = unityContainer.Resolve<INavigationService>();
            navigationService.Navigate<DisplayWorkshopsViewModel>();
        }

        private UnityContainer InitializeDependencyContainer()
        {
            var unityContainer = new UnityContainer();

            // register navigation frame
            unityContainer.RegisterType<INavigationFrame, ShellViewModel>(new ContainerControlledLifetimeManager());

            // register view model factory
            unityContainer.RegisterType<IViewModelFactory, ViewModelFactory>(new ContainerControlledLifetimeManager());

            // register navigation service
            unityContainer.RegisterType<INavigationService, NavigationService>(new ContainerControlledLifetimeManager());

            // register unit of work (new instance every time)
            unityContainer.RegisterType<IUnitOfWorkFactory, UnitOfWorkFactory>();

            // initialize and register validation manager
            var validationManager = InitializeValidationManager();
            unityContainer.RegisterInstance<IValidationManager>(validationManager);

            return unityContainer;
        }

        private IValidationManager InitializeValidationManager()
        {
            var rules = new List<IValidationRule>();

            // business rules
            var workshopRule = new WorkshopValidationRule();
            rules.Add(workshopRule);

            var attendeeRule = new AttendeeValidationRule();
            rules.Add(attendeeRule);

            var validationManager = new ValidationManager(rules);
            return validationManager;
        }
    }
}
