﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Workshops.Model;

namespace Workshops.ViewModels
{
    public class EditWorkshopViewModel : ViewModelBase
    {
        private Workshop _workshop;
        private Attendee _selectedAttendee;
        private string _newAttendeeFirstName;
        private string _newAttendeeLastName;
        private ObservableCollection<Attendee> _attendees;

        public Workshop Workshop
        {
            get
            {
                return _workshop;
            }
            set
            {
                _workshop = value;
                RaisePropertyChanged("Workshop");
            }
        }

        public ObservableCollection<Attendee> Attendees
        {
            get
            {
                return _attendees;
            }
            set
            {
                _attendees = value;
                RaisePropertyChanged("Attendees");
            }
        }

        public Attendee SelectedAttendee
        {
            get
            {
                return _selectedAttendee;
            }
            set
            {
                _selectedAttendee = value;
                RaisePropertyChanged("SelectedAttendee");
            }
        }

        public string NewAttendeeFirstName
        {
            get
            {
                return _newAttendeeFirstName;
            }
            set
            {
                _newAttendeeFirstName = value;
                RaisePropertyChanged("NewAttendeeFirstName");
            }
        }

        public string NewAttendeeLastName
        {
            get
            {
                return _newAttendeeLastName;
            }
            set
            {
                _newAttendeeLastName = value;
                RaisePropertyChanged("NewAttendeeLastName");
            }
        }

        public ICommand AddNewAttendeeCommand
        {
            get;
            private set;
        }

        public ICommand DeleteSelectedAttendeeCommand
        {
            get;
            private set;
        }

        public ICommand SaveCommand
        {
            get;
            private set;
        }

        public ICommand CancelCommand
        {
            get;
            private set;
        }

        public EditWorkshopViewModel(Workshop workshop)
        {
            AddNewAttendeeCommand = new RelayCommand(AddNewAttendee);
            DeleteSelectedAttendeeCommand = new RelayCommand(DeleteSelectedAttendee);
            SaveCommand = new RelayCommand(Save);
            CancelCommand = new RelayCommand(Cancel);

            // prepare workshop
            if (workshop == null)
            {
                throw new ArgumentNullException("workshop");
            }
            Workshop = workshop;

            // prepare attendees
            Attendees = new ObservableCollection<Attendee>(Workshop.Attendees);
        }

        private void AddNewAttendee()
        {
            if (string.IsNullOrWhiteSpace(NewAttendeeFirstName))
            {
                MessageBox.Show("Please enter a first name.");
                return;
            }

            if (string.IsNullOrWhiteSpace(NewAttendeeLastName))
            {
                MessageBox.Show("Please enter a last name.");
                return;
            }

            if (NewAttendeeFirstName.Length > 30)
            {
                MessageBox.Show("The maximum length of first names is 30 characters.");
                return;
            }

            if (NewAttendeeLastName.Length > 30)
            {
                MessageBox.Show("The maximum length of last names is 30 characters.");
                return;
            }

            // create attendee
            var newAttendee = new Attendee();
            newAttendee.FirstName = NewAttendeeFirstName;
            newAttendee.LastName = NewAttendeeLastName;

            // add to list
            Attendees.Add(newAttendee);

            // clean display
            NewAttendeeFirstName = string.Empty;
            NewAttendeeLastName = string.Empty;
        }

        private void DeleteSelectedAttendee()
        {
            if (SelectedAttendee == null)
            {
                MessageBox.Show("Please select an attendee you want to remove.");
                return;
            }

            // remove from list
            Attendees.Remove(SelectedAttendee);
        }

        private void Save()
        {
            if (string.IsNullOrWhiteSpace(Workshop.Title))
            {
                MessageBox.Show("A workshop must have a title.");
                return;
            }

            if (Workshop.EventDate < DateTime.Now)
            {
                MessageBox.Show("A workshop date must be in the future.");
                return;
            }

            using (var context = new WorkshopsDatabaseContext())
            {
                // attach attendees
                foreach (var attendee in Attendees)
                {
                    // attach to context
                    context.Attendees.Attach(attendee);
                    var attendeeEntry = context.Entry(attendee);
                    
                    // set correct state
                    if (attendee.Id == 0)
                    {
                        // new item
                        attendeeEntry.State = EntityState.Added;
                        Workshop.Attendees.Add(attendee);
                    }
                    else
                    {
                        // edited
                        attendeeEntry.State = EntityState.Modified;
                    }
                }

                // make sure to treat deleted attendees correctly
                var deletedAttendees = Workshop.Attendees
                                               .Where(a => !Attendees.Contains(a))
                                               .ToList();
                foreach (var deletedAttendee in deletedAttendees)
                {
                    // remove from workshop
                    Workshop.Attendees.Remove(deletedAttendee);

                    // attach and mark for deletion
                    context.Attendees.Attach(deletedAttendee);
                    var deletedAttendeeEntry = context.Entry(deletedAttendee);
                    deletedAttendeeEntry.State = EntityState.Deleted;
                }

                // attach workshop
                context.Workshops.Attach(Workshop);

                // get entry
                var workshopEntry = context.Entry(Workshop);
                workshopEntry.State = Workshop.Id > 0 ? EntityState.Modified : EntityState.Added;

                // save
                context.SaveChanges();
            }

            NavigateToOverviewPage();
        }

        private void Cancel()
        {
            if (MessageBox.Show("Are you sure?", "Cancel", MessageBoxButton.YesNo) != MessageBoxResult.Yes)
            {
                return;
            }

            NavigateToOverviewPage();
        }

        private static void NavigateToOverviewPage()
        {
            // create overview view model
            var displayWorkshopsVM = new DisplayWorkshopsViewModel();

            // navigate!
            var app = Application.Current as App;
            if (app != null)
            {
                app.ShellViewModel.CurrentPage = displayWorkshopsVM;
            }
        }
    }

}
